Sub Main()

    videoMode=CreateObject("roVideoMode")
    videoMode.SetGraphicsZOrder("back")

    audioOutput = CreateObject("roAssociativeArray")
    audioOutput["HDMI"] = CreateObject("roAudioOutput", "HDMI")
    audioOutput["Analog"] = CreateObject("roAudioOutput", "Analog")
    audioOutput["SPDIF"] = CreateObject("roAudioOutput", "SPDIF")
    audioOutput["USB"] = CreateObject("roAudioOutput", "USB")
    audioOutput["NONE"] = CreateObject("roAudioOutput", "NONE")

    systemTime=CreateObject("roSystemTime")
    screenRegistrySection = CreateObject("roRegistrySection", "screen")
    debugRegistrySection = CreateObject("roRegistrySection", "debug")
    debugPort = 3000

    registrySection = createobject("roregistrysection", "html")
    multiProcess = registrySection.read("mp")
    if multiProcess <> "1" then
        registrySection.write("mp","1")
        registrySection.flush()
        RebootSystem()
    endif

    msgPort = CreateObject("roMessagePort")
    fullScreenRectangle = CreateObject("roRectangle", 0, 0, 1200, 1920)

    currentTransform = GetCurrentTransform(screenRegistrySection)
    config = {
        nodejs_enabled: true
        brightsign_js_objects_enabled: true
        focus_enabled: true
        javascript_enabled: true
        url: "file:///index.html"
        storage_path: "SD:"
        storage_quota: 1073741824
        transform: currentTransform
        hwz_default: "on"
        'port: msgPort ' Doesn't work in 6.2 FW. Used this htmlWidget.SetPort(msgPort)
    }

    if debugRegistrySection.Read("native") = "1" then
        config["inspector_server"] = {
            port: debugPort
        }
    endif

    htmlWidget = CreateObject("roHtmlWidget", fullScreenRectangle, config)
    htmlWidget.SetPort(msgPort)
    sleep(5000)
    htmlWidget.Show()

    htmlWidget.PostJSMessage({
        type: "ready"
    })

    videoPlayers = CreateObject("roAssociativeArray")
    videoPlayerPorts = CreateObject("roAssociativeArray")

    while true
        htmlMessage = wait(50, msgPort)
        if type(htmlMessage) = "roHtmlWidgetEvent" then
            print "type(htmlMessage)=";type(htmlMessage)
            ProcessHtmlWidgetEvent(htmlMessage, htmlWidget, videoMode, audioOutput, systemTime, screenRegistrySection, debugRegistrySection, videoPlayers, videoPlayerPorts)
        endif

        while videoPlayerPorts.IsNext()
            videoPlayerPortKey = videoPlayerPorts.Next()
            videoPlayerPort = videoPlayerPorts.Lookup(videoPlayerPortKey)
            videoMessage = wait(50, videoPlayerPort)
            if type(videoMessage) = "roVideoEvent" then
                print "video event ";videoMessage.GetInt()
                ProcessVideoEvent(videoMessage, videoPlayerPortKey, htmlWidget)
            endif
        end while
        videoPlayerPorts.Reset()
    end while

End Sub

Function ProcessHtmlWidgetEvent(msg, htmlWidget, videoMode, audioOutput, systemTime, screenRegistrySection, debugRegistrySection, videoPlayers, videoPlayerPorts)
    eventData = msg.GetData()
    if type(eventData) = "roAssociativeArray" and type(eventData.reason) = "roString" then
        print "reason = ";eventData.reason
        response = CreateObject("roAssociativeArray")
        if eventData.reason = "message" then
            if eventData.message.type = "system.reboot" then
                RebootSystem()
                response["succeeded"] = true
            endif
            if eventData.message.type = "application.restart" then
                RestartApplication()
                'RestartScript() 'soft restart of application
                response["succeeded"] = true
            endif
            if eventData.message.type = "display.power_on" then
                videoMode.SetPowerSaveMode(false)
                videoMode.EnableVideo(true)
                response["succeeded"] = true
            endif
            if eventData.message.type = "display.power_off" then
                videoMode.SetPowerSaveMode(true)
                videoMode.EnableVideo(false)
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.capture" then
                screenshotOptions = CreateObject("roAssociativeArray")
                screenshotOptions["filename"] = eventData.message.filePath
                screenshotOptions["width"] = eventData.message.width
                screenshotOptions["height"] = eventData.message.height
                screenshotOptions["filetype"] = eventData.message.fileType
                screenshotOptions["quality"] = eventData.message.quality
                screenshotOptions["rotation"] = eventData.message.rotation
                screenshotOptions["async"] = 0
                videoMode.Screenshot(screenshotOptions)
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.set_brightness" then
                graphicsColorValues = CreateObject("roAssociativeArray")
                graphicsColorValues["brightness"] = eventData.message.brightness
                videoMode.AdjustGraphicsColor(graphicsColorValues)
                response["succeeded"] = true
            endif
            if eventData.message.type = "audio.set_volume" then
                audioOutput["HDMI"].SetVolume(eventData.message.volume)
                audioOutput["Analog"].SetVolume(eventData.message.volume)
                audioOutput["SPDIF"].SetVolume(eventData.message.volume)
                audioOutput["USB"].SetVolume(eventData.message.volume)
                audioOutput["NONE"].SetVolume(eventData.message.volume)
                response["succeeded"] = true
            endif
            if eventData.message.type = "date_time.set_current_time_with_timezone" then
                currentDateTime = CreateObject("roDateTime")
                currentDateTime.FromIsoString(eventData.message.formatedIsoString)
                systemTime.SetTimeZone(eventData.message.timezone)
                systemTime.SetUtcDateTime(currentDateTime)
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.set_orientation" then
                screenRegistrySection.Write("orientation", eventData.message.orientation)
                screenRegistrySection.Flush()
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.get_orientation" then
                response["orientation"] = screenRegistrySection.Read("orientation")
                response["transform"] = GetCurrentTransform(screenRegistrySection)
                response["succeeded"] = true
            endif
            if eventData.message.type = "debug.set_debug" then
                debugRegistrySection.Write("native", eventData.message.enabled)
                debugRegistrySection.Flush()
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.init_player" then
                videoPlayerKey = eventData.message.key
                videoPlayerPort = CreateObject("roMessagePort")
                videoPlayer = CreateObject("roVideoPlayer")
                videoPlayer.SetPort(videoPlayerPort)
                videoPlayers.AddReplace(videoPlayerKey, videoPlayer)
                videoPlayerPorts.AddReplace(videoPlayerKey, videoPlayerPort)
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.play" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                videoX = strtoi(eventData.message.x)
                videoY = strtoi(eventData.message.y)
                videoWidth = strtoi(eventData.message.width)
                videoHeight = strtoi(eventData.message.height)
                videoOrientation = GetCurrentTransform(screenRegistrySection)
                if eventData.message.vtype = "video" then
                    videoFilePath = eventData.message.filePath
                    PlayVideo(videoPlayer, videoFilePath, videoX, videoY, videoWidth, videoHeight, videoOrientation)
                else if eventData.message.vtype = "stream" then
                    streamUri = eventData.message.uri
                    PlayStream(videoPlayer, streamUri, videoX, videoY, videoWidth, videoHeight, videoOrientation)
                else if eventData.message.vtype = "hdmi" then
                    PlayHDMI(videoPlayer, videoX, videoY, videoWidth, videoHeight, videoOrientation)
                endif
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.stop" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                videoPlayer.StopClear()
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.pause" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                videoPlayer.Pause()
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.resume" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                videoPlayer.Resume()
                response["succeeded"] = true
            endif
            response["type"] = "response"
            response["invocationUid"] = eventData.message.invocationUid
            htmlWidget.PostJSMessage(response)
        endif
        if eventData.reason = "load-error" then
            print "message = ";eventData.message
        endif
    endif
End Function

Function ProcessVideoEvent(msg, videoKey, htmlWidget)
    videoEvent = CreateObject("roAssociativeArray")
    videoEvent["type"] = "video_event"
    videoEvent["key"] = videoKey
    videoEvent["eventType"] = msg.GetInt()
    htmlWidget.PostJSMessage(videoEvent)
End Function

Function PlayVideo(videoPlayer, filePath, x, y, width, height, orientation)
    rectangle = CreateObject("roRectangle", x, y, width, height)
    videoPlayer.SetRectangle(rectangle)
    videoPlayer.SetTransform(orientation)
    videoPlayer.PlayFile(filePath)
End Function

Function PlayStream(videoPlayer, uri, x, y, width, height, orientation)
    rectangle = CreateObject("roRectangle", x, y, width, height)
    videoPlayer.SetRectangle(rectangle)
    videoPlayer.SetTransform(orientation)
    stream = CreateObject("roRtspStream", uri)
    videoPlayer.PlayFile({ rtsp: stream })
End Function

Function PlayHDMI(videoPlayer, x, y, width, height, orientation)
    rectangle = CreateObject("roRectangle", x, y, width, height)
    videoPlayer.SetRectangle(rectangle)
    videoPlayer.SetTransform(orientation)
    videoInput = CreateObject("roVideoInput")
    videoPlayer.PlayFile(videoInput)
End Function

Function GetCurrentTransform(screenRegistrySection)
    orientation = screenRegistrySection.Read("orientation")
    if orientation = "LANDSCAPE" then
        return "identity"
    endif
    if orientation = "PORTRAIT" then
        return "rot90"
    endif
    if orientation = "PORTRAIT_FLIPPED" then
        return "rot270"
    endif
    return "identity"
End Function
